//
//  TLFlipTransition.m
//  TLTransition
//
//  Created by Tim Lai on 2012/2/20.

// This code is distributed under the terms and conditions of the MIT license. 

// Copyright (c) 2012 Tim Lai
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import "TLFlipTransition.h"

@implementation TLFlipTransition

- (void)initTransition {
    if (backgroundLayer) {
        [flipLayer removeFromSuperlayer];
        [backgroundLayer removeFromSuperlayer];
    }
    
    CGRect rect = self.rootLayer.bounds;
	rect.size.width /= 2;
	
	backgroundLayer = [CALayer layer];
	backgroundLayer.frame = self.rootLayer.bounds;
	backgroundLayer.zPosition = -300000;
    
	CALayer *leftLayer = [CALayer layer];
	leftLayer.frame = rect;
	leftLayer.masksToBounds = YES;
	leftLayer.contentsGravity = kCAGravityLeft;
	
	[backgroundLayer addSublayer:leftLayer];
	
	rect.origin.x = rect.size.width;
	
	CALayer *rightLayer = [CALayer layer];
	rightLayer.frame = rect;
	rightLayer.masksToBounds = YES;
	rightLayer.contentsGravity = kCAGravityRight;
	
	[backgroundLayer addSublayer:rightLayer];
	
	if (self.directionType == TLDirectionRight) {
		leftLayer.contents = (id) [self.endImage CGImage];
		rightLayer.contents = (id) [self.beginImage CGImage];
	} else {
		leftLayer.contents = (id) [self.beginImage CGImage];
		rightLayer.contents = (id) [self.endImage CGImage];
	}
    
	[self.rootLayer  addSublayer:backgroundLayer];
	
	rect.origin.x = 0;
	
	flipLayer = [CATransformLayer layer];
	flipLayer.anchorPoint = CGPointMake(1.0, 0.5);
	flipLayer.frame = rect;
	
	[self.rootLayer addSublayer:flipLayer];
	
	CALayer *backLayer = [CALayer layer];
	backLayer.frame = flipLayer.bounds;
	backLayer.doubleSided = NO;
	backLayer.masksToBounds = YES;
	
	[flipLayer addSublayer:backLayer];
	
	CALayer *frontLayer = [CALayer layer];
	frontLayer.frame = flipLayer.bounds;
	frontLayer.doubleSided = NO;
	frontLayer.masksToBounds = YES;
	frontLayer.transform = CATransform3DMakeRotation(M_PI, 0, 1.0, 0);
	
	[flipLayer addSublayer:frontLayer];
	
	if (self.directionType == TLDirectionRight) {
		backLayer.contents = (id) [self.beginImage CGImage];
		backLayer.contentsGravity = kCAGravityLeft;
		
		frontLayer.contents = (id) [self.endImage CGImage];
		frontLayer.contentsGravity = kCAGravityRight;
		
		CATransform3D transform = CATransform3DMakeRotation(0.0, 0.0, 1.0, 0.0);
		transform.m34 = 1.0f / 2500.0f;
		
		flipLayer.transform = transform;
		
		currentAngle = startFlipAngle = 0;
		endFlipAngle = -M_PI;
	} else {
		backLayer.contentsGravity = kCAGravityLeft;
		backLayer.contents = (id) [self.endImage CGImage];
		
		frontLayer.contents = (id) [self.beginImage CGImage];
		frontLayer.contentsGravity = kCAGravityRight;
		
		CATransform3D transform = CATransform3DMakeRotation(-M_PI / 1.1, 0.0, 1.0, 0.0);
		transform.m34 = 1.0f / 2500.0f;
		
		flipLayer.transform = transform;
		
		currentAngle = startFlipAngle = -M_PI;
		endFlipAngle = 0;
	}
}

    
- (void)drawContentAtProgress:(float)progress {
    float newAngle = startFlipAngle + progress * (endFlipAngle - startFlipAngle);
	
	currentAngle = newAngle;
	
	CATransform3D endTransform = CATransform3DIdentity;
	endTransform.m34 = 1.0f / 2500.0f;
	endTransform = CATransform3DRotate(endTransform, newAngle, 0.0, 1.0, 0.0);	
	
	flipLayer.transform = endTransform;
    
}

@end
